@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007-2021 Free Software Foundation, Inc.
+   Copyright (C) 2007-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -94,6 +94,16 @@ assoc_elem_link (mu_assoc_t assoc, int idx)
     assoc->head = assoc->tab[idx];
   assoc->tail = assoc->tab[idx];
 }
+
+static void
+assoc_elem_link_at_head (mu_assoc_t assoc, int idx)
+{
+  assoc->tab[idx]->prev = NULL;
+  if (assoc->head)
+    assoc->head->prev = assoc->tab[idx];
+  assoc->tab[idx]->next = assoc->head;
+  assoc->head = assoc->tab[idx];
+}
 
 static unsigned
 hash_dfl (const char *name, unsigned long hash_num)
@@ -467,6 +477,12 @@ mu_assoc_install_ref2 (mu_assoc_t assoc, const char *name,
   *(void**)ret_val = &assoc->tab[idx]->data;
   if (ret_name)
     *ret_name = assoc->tab[idx]->name;
+
+  if ((assoc->flags & MU_ASSOC_LRU) && assoc->tab[idx]->prev)
+    {
+      assoc_elem_unlink (assoc, idx);
+      assoc_elem_link_at_head (assoc, idx);
+    }
   
   return inst ? 0 : MU_ERR_EXISTS;
 }  
@@ -921,27 +937,43 @@ mu_assoc_tail_set_mark (mu_assoc_t asc, int mark)
 }
 
 int
-mu_assoc_pop (mu_assoc_t asc, char const *name, void *ret_val)
+mu_assoc_pop (mu_assoc_t asc, char **ret_name, void *ret_val)
 {
-  if (!asc || !name)
+  if (!asc)
     return EINVAL;
 
-  if (asc->tail && ret_val != NULL)
+  if (asc->tail)
     {
-      *(void**)ret_val = asc->tail->data;
+      if (ret_name)
+	{
+	  char *p = strdup (asc->tail->name);
+	  if (!p)
+	    return errno;
+	  *ret_name = p;
+	}
+      if (ret_val != NULL)
+	*(void**)ret_val = asc->tail->data;
     }
   return assoc_remove_elem (asc, asc->tail, ret_val != NULL);
 }
 
 int
-mu_assoc_shift (mu_assoc_t asc, char const *name, void *ret_val)
+mu_assoc_shift (mu_assoc_t asc, char **ret_name, void *ret_val)
 {
-  if (!asc || !name)
+  if (!asc)
     return EINVAL;
 
-  if (asc->head && ret_val != NULL)
+  if (asc->head)
     {
-      *(void**)ret_val = asc->head->data;
+      if (*ret_name)
+	{
+	  char *p = strdup (asc->head->name);
+	  if (!p)
+	    return errno;
+	  *ret_name = p;
+	}
+      if (ret_val != NULL)
+	*(void**)ret_val = asc->head->data;
     }
   return assoc_remove_elem (asc, asc->head, ret_val != NULL);
 }
