@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,5 +25,5 @@
 int
 mu_imapio_send_msgset (mu_imapio_t io, mu_msgset_t msgset)
 {
-  return mu_msgset_print (io->_imap_stream, msgset);
+  return mu_msgset_imap_print (io->_imap_stream, msgset);
 }
