@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008-2020 Free Software Foundation, Inc.
+   Copyright (C) 2008-2025 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -46,7 +46,7 @@
 #include <mailutils/url.h>
 #include <mailutils/util.h>
 
-typedef RETSIGTYPE (*mu_sig_handler_t) (int);
+typedef void (*mu_sig_handler_t) (int);
 
 static mu_sig_handler_t
 set_signal (int sig, mu_sig_handler_t handler)
@@ -57,7 +57,11 @@ set_signal (int sig, mu_sig_handler_t handler)
     act.sa_handler = handler;
     sigemptyset (&act.sa_mask);
     act.sa_flags = 0;
-    sigaction (sig, &act, &oldact);
+    if (sigaction (sig, &act, &oldact))
+      {
+	mu_error ("sigaction %d: %s", sig, mu_strerror (errno));
+	return SIG_DFL;
+      }
     return oldact.sa_handler;
   }
 #else
@@ -80,12 +84,14 @@ struct _mu_m_server
   mu_m_server_preflight_fp preflight; /* Pre-flight check function */
   mu_m_server_handler_fp conn;   /* Connection handler function. */
   mu_m_server_handler_fp prefork;/* Pre-fork function. */
+  mu_m_server_config_default_fp config_default;
+
   void *data;                    /* User-supplied data for conn and prefork. */
   size_t app_data_size;
   
   int mode;                      /* Server mode: should be removed. */
   
-  int foreground;                /* Should the server remain in foregorund? */
+  int foreground;                /* Should the server remain in foreground? */
   size_t max_children;           /* Maximum number of sub-processes to run. */
   size_t num_children;           /* Current number of running sub-processes. */
   pid_t *child_pid;
@@ -96,7 +102,8 @@ struct _mu_m_server
   
   sigset_t sigmask;              /* A set of signals to handle by the
 				    m-server.  */
-  mu_sig_handler_t sigtab[NSIG]; /* Keeps old signal handlers. */
+  mu_sig_handler_t sigtab[NSIG+1];/* Keeps old signal handlers. sigtab[0] is
+				     not used. */
   const char *(*strexit) (int);  /* Convert integer exit code to textual
 				    description. */
 };
@@ -236,7 +243,7 @@ mu_m_server_idle (void *server_data MU_ARG_UNUSED)
   return stop;
 }
 
-static RETSIGTYPE
+static void
 m_srv_signal (int signo)
 {
   switch (signo)
@@ -330,6 +337,12 @@ mu_m_server_set_prefork (mu_m_server_t srv, mu_m_server_handler_fp fun)
   srv->prefork = fun;
 }
 
+void
+mu_m_server_set_config_default (mu_m_server_t srv, mu_m_server_config_default_fp fun)
+{
+  srv->config_default = fun;
+}
+
 void
 mu_m_server_set_data (mu_m_server_t srv, void *data)
 {
@@ -475,10 +488,12 @@ mu_m_server_listen (mu_m_server_t msrv, struct mu_sockaddr *s, int type)
       mu_error ("%s", mu_strerror (ENOMEM));
       exit (1);
     }
+  pconf->timeout = msrv->timeout;
+  pconf->single_process = 0;
+  if (msrv->config_default)
+    msrv->config_default (pconf, msrv->data);
   pconf->msrv = msrv;
   pconf->tcpsrv = tcpsrv;
-  pconf->single_process = 0;
-  pconf->timeout = msrv->timeout;
   MU_ASSERT (mu_ip_server_set_data (tcpsrv, pconf, mu_srv_config_free));
   if (!msrv->srvlist)
     MU_ASSERT (mu_list_create (&msrv->srvlist));
@@ -545,7 +560,7 @@ mu_m_server_begin (mu_m_server_t msrv)
 		  mu_strerror (rc));
       }
 
-  for (i = 0; i < NSIG; i++)
+  for (i = 1; i < NSIG; i++)
     if (sigismember (&msrv->sigmask, i))
       msrv->sigtab[i] = set_signal (i, m_srv_signal);
 }
@@ -554,8 +569,8 @@ void
 mu_m_server_restore_signals (mu_m_server_t msrv)
 {
   int i;
-  
-  for (i = 0; i < NSIG; i++)
+
+  for (i = 1; i < NSIG; i++)
     if (sigismember (&msrv->sigmask, i))
       set_signal (i, msrv->sigtab[i]);
 }
