@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2021 Free Software Foundation, Inc.
+   Copyright (C) 1999-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -806,6 +806,14 @@ action_pack (void)
       mu_error (_("cannot read input mailbox: %s"), mu_strerror (errno));
       return 1;
     }
+
+  if (count == 0)
+    {
+      mu_mailbox_close (mbox);
+      mu_mailbox_destroy (&mbox);
+      return 0;
+    }
+  
   pack_tab = mu_calloc (count, sizeof pack_tab[0]); /* Never freed. No use to
 		 				       try to. */
 
